' === LCD08x2-CustCharDriver14M2new.bas ============== 2015.09.04 ===

' This program runs on a Picaxe-14M2 @ 32MHz. It receives serial data
' from a master processor, and diplays it on an HD44780-type 8x2 LCD.
' The incomming data must be sent at 4800 baud (inverted).

' Because we're using DB5 - DB2, rather than DB7 - DB4, high nibbles
' need to be divided by 4, & low nibbles need to be multiplied by 4,
' in order to place the data on the correct pins.

' This driver can process serial input of any length, up to a maximum
' of 20 bytes. Exceeding the limit may cause the program to misbehave

' === Directives ========
  #com 3                      'specify COM port for downloading
  #picaxe 14M2                'specify PICAXE processor
  #no_data                    'comment-out when updating data
  #terminal off               'disable terminal window
  
' === Constants =========
  symbol  baud = N4800_32     'data must be transmitted at 4800 baud 
  symbol enPin = B.1          'LCD enable pin on B.1
  symbol rxPin = C.4          'serial data received on C.4
  symbol   cmd = 0            'RSbit = 0 (low) for command bytes
  symbol   txt = 1            'RSbit = 1 (high) for text bytes 

' === Variables =========
  symbol RSpin = pinB.0       'the current state of pin B.0
  symbol flagz = b0           'reserved for binary variables
  symbol RSbit = bit0         'used to set up for cmnd/text byte
  symbol  bite = b1           'byte to sent to LCD (can't use "byte") 
  symbol index = b2           'used in for/next loop
  symbol hiNib = b3           'the high nibble of char
  symbol loNib = b4           'the low nobble of char
                              
' === Data ============
' Data 0; Lowercase "g"       'don't change the order of the chars
  data (%00000)
  data (%00000)
  data (%01111) '  ****
  data (%10001) ' *   *
  data (%10001) ' *   *
  data (%01111) '  ****
  data (%00001) '     *
  data (%01110) '  ***

' Data 8; Lowercase "p"
  data (%00000)
  data (%00000)
  data (%10110) ' * **
  data (%11001) ' **  *
  data (%10001) ' *   *
  data (%11110) ' ****
  data (%10000) ' *
  data (%10000) ' *

' Data 16; Lowercase "q"
  data (%00000)
  data (%00000)
  data (%01101) '  ** *	
  data (%10011) ' *  **	
  data (%10001) ' *   *
  data (%01111) '  ****
  data (%00001) '     *	
  data (%00001) '     *

' Data 24; Lowercase "j"	
  data (%00010) '    *
  data (%00000) '    
  data (%00110) '   **
  data (%00010) '    *	
  data (%00010) '    *
  data (%00010) '    *
  data (%10010) ' *  *	
  data (%01100) '  **	

' Data 32; Lowercase "y"	
  data (%00000)
  data (%00000)
  data (%10001) ' *   *
  data (%10001) ' *   *	
  data (%10001) ' *   *
  data (%01111) '  ****	
  data (%00001) '     *	
  data (%01110) '  ***
	
' Data 40; Backslash
  data (%00000)
  data (%10000) ' *
  data (%01000) '  *
  data (%00100) '   *
  data (%00010) '    *
  data (%00001) '     *
  data (%00000)
  data (%00000)

' Data 48; Bullet
  data (%00000)
  data (%01110) '  ***
  data (%11111) ' *****
  data (%11111) ' *****
  data (%11111) ' *****
  data (%01110) '  ***
  data (%00000)
  data (%00000)	
	
' Data 56; Degree Sign
  data (%01100) '  **
  data (%10010) ' *  *
  data (%10010) ' *  *
  data (%01100) '  ** 
  data (%00000)
  data (%00000)
  data (%00000)
  data (%00000)
 
#rem
' Include the following two data statements to test the driver:
' Comment them out when the code is working.
  data("jpigquay")            'Data 64
  data("Set 225~")            'Data 72
#endrem
  
' === Begin Main Program ============================================
setfreq m32                   'run as fast as possible
dirsB = %11111111             'all PortB pins are outputs
dirsC = %11000111             'C.3, C.4, and C.5 are inputs

gosub initLCD
gosub clearStorage

#rem
'Demonstrate that the code works - comment out when sure.
bite = 1                      'instruction: clear display
gosub sendToLCD               'send instruction to LCD
pause 32                      'minimum 2mS delay for clear display
 	
bite = 128                    'instruction: go to start of 1st row
gosub sendToLCD               'send instruction to LCD

for index = 64 to 71          'output data to 1st row of LCD
  read index, bite            'read character from EEPROM
  gosub sendToLCD             'send character to LCD
next index
	
bite = 192                    'instruction: go to start of 2nd row
gosub sendToLCD               'send instruction to LCD

for index = 72 to 79          'output data to 2nd row of LCD
  read index, bite            'read character from EEPROM
  gosub sendToLCD             'send character to LCD
next index

end
#endrem
  
' === Main loop: Receive data and display it. =======================
do  
  bptr =28                    'begin storing data at location 28

  serin rxPin,baud,@bptrinc   'no timeout; just wait for first char
  
'This looks a little silly, but a loop wouldn't be fast enough.
serin [800],RxPin,baud,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc,@bptrinc

  bptr = 28                   'Reset byte pointer to first location.

  do until @bptr = 0          'loop until we reach the end of input
    bite = @bptrinc           'get each byte
    gosub sendToLCD           'send it to the LCD
    if RSbit = cmd then       'if last byte was a command,
      pause 32                'allow time for it to be implemented
    endif
  loop                        'end of inner do/loop

  gosub clearStorage
loop                          'end of main do/loop
	
' === End Main Program - Subroutines Follow =========================
clearStorage:                 'store 0 in first 23 storage bytes
  for bptr = 28 to 50
    poke bptr,0
  next bptr
return

initLCD:                      'initialize the LCD
  outpinsB = 0                'clear all output lines
  pause 6400                  'pause 200 mS for LCD initialization

  outpinsB = 12               '(48/4=12) set to 4-bit operation
  pulsout enPin,8             'send data
  pause 320                   'pause 10 mS
  pulsout enPin,8             'send again
  pulsout enPin,8             'send again (necessary in 4-bit mode)
	
  outpinsB = 8       	        '(32/4=8) set to 4-bit operation
  pulsout enPin,8             'send data.
  pulsout enPin,8             'send again (necessary in 4-bit mode)
	
  outpinsB = 32               '(128/4=32) set to 2 line operation
  pulsout enPin,8             'send data

  bite  = 12                  'display on, cursor off
  RSbit = cmd
  gosub sendToLCD2            'send instruction to LCD
	
' Load custom chars into CGRAM
  bite  = 64                  'command: define custom characters
  RSbit = cmd
  gosub sendToLCD2
	
  RSbit = txt
  for index = 0 to 63         'install 8 custom characters
    read index, bite
    gosub sendToLCD2
  next index

  bite  = 2                   'command: cursor home
  RSbit = cmd
  gosub sendToLCD2
return

sendToLCD:                    'Output 1 byte to the LCD display
  RSbit = txt                 'text byte is the default
  
  select case bite
    case 7                    'for the degree sign
      RSbit = txt
    case 0 to 31
      RSbit = cmd             'set up for command byte
    case 128 to 207           'commands for moving the cursor
      RSbit = cmd             'to any on-screen location
    case 103,106,112,113      '"g", "j", "p", & "q"
      bite = bite - 103
    case 121                  '"y"
      bite = 4
    case 124                  'replace "|" with degree sign
      bite = 7
  endselect
	
sendToLCD2:
  hiNib = bite & %11110000    'isolate high nibble of char
  hiNib = hiNib / 4           'move to pins 5,4 ,3 & 2
  outpinsB = hiNib            'place high nibble of char onto pins
  RSpin = RSbit               'RSpin = 0 for cmnd or 1 for text
  pulsout enPin, 1            'send data

  loNib = bite & %00001111    'isolate low nibble of char	
  loNib = loNib * 4           'move to pins 5,4 ,3 & 2
  outpinsB = loNib            'place low nibble of char onto pins
  RSpin = RSbit               'RSpin = 0 for cmnd or 1 for text
  pulsout enPin, 1            'send data
return